; qmeteors -	Travel through the meteor storm, attempting to get supplies
;		to the colony on the other side. Once there, you will need
;		make your way back.

;		Within the storm, look out for old wreaks which can
;		be collected for more points.

;		There can also be found artefacts left by an alien race
;		that can be used to destroy all meteors in range.

;		Watch out, the storm is getting quicker.

;		A lot of the techniques given by Johan "Dr Beep" Koelman
;		in his PDF on fully using the 1K ZX81.

;		Fixed to work with Editions 1 and 2 of the ROM.

; 		Keys:
;			Left: 		Q - T
;			Right:		Y - P
;			Smartbomb:	B - Space

; Features:
;	Hi-Score
;	Different sized meteors
;	Collect savage
;	Three lives
;	Return journey
;	power-up - smartbomb (B)
;	Speedup each journey

;  ____0___1___2___3___4___5___6___7___8___9___A___B___C___D___E___F____
;  00 SPC #_  _#  ##  __  #_  _#  ##  HH  __  HH   "  GBP  $   :   ?  0F
;         __  __  __  #_  #_  #_  #_  HH  HH  __

;  10  (   )   >   <   =   +   -   *   /   ;   ,   .   0   1   2   3  1F
;  20  4   5   6   7   8   9   A   B   C   D   E   F   G   H   I   J  2F
;  30  K   L   M   N   O   P   Q   R   S   T   U   V   W   X   Y   Z  3F

; Keyboard matrix when reading LASTK

; 
; LASTK+1         FD  FB  F7  EF  DF | DF  EF  F7  FB  FD
;                                    |
; LASTK    BIT                       |                      BIT
; F7        3     1   2   3   4   5  | 6   7   8   9   0     4   EF
;                                    |
; FB        2     Q   W   E   R   T  | Y   U   I   O   P     5   DF
;                                    |
; FD        1     A   S   D   F   G  | H   J   K   L  ret    6   BF
;                                    |
; FE        0         Z   X   C   V  | B   N   M   .  spc    7   7F

; Shift is bit 0 on LASTK + 1.

	.INCLUDE "zx81_1k.inc"

.SECTION "PFILE"

	.INCLUDE "zx81_loader.inc"

;*******************************************************************************
; SYSVARS available for reuse

	;.EQU		$4000		; ERR_NR
	;.EQU		$4001		; FLAGS
	;.EQU		$4002		; ERR_SP
	.EQU	GAMEOVR	$4003		; ERR_SP + 1
	;.EQU	   "	$4004		; RAMTOP
	;.EQU	   "	$4005		; RAMTOP + 1
	;.EQU	   "	$4006		; MODE
	;.EQU	   "	$4007		; PPC
	;.EQU	   "	$4008		; PPC + 1
	;.EQU	   "	$4009		; VERSN
	;.EQU	   "	$400A		; E_PPC
	;.EQU	   "	$400B		; E_PPC + 1

	; 400C/400D - DFILE

	.EQU	ANYKEY	$400E		; DF_CC
	;.EQU	   "	$400F		; DF_CC + 1
	;.EQU	   "	$4010		; VARS
	;.EQU	   "	$4011		; VARS + 1
	;.EQU	   "	$4012		; DEST
	;.EQU	   "	$4013		; DEST + 1
	;.EQU	   "	$4014		; E_LINE
	;.EQU	   "	$4015		; E_LINE + 1
	;.EQU	   "	$4016		; CH_ADD
	.EQU	MADEIT	$4017		; CH_ADD + 1
	;.EQU	   "	$4018		; X_PTR
	;.EQU	   "	$4019		; X_PTR + 1
	;.EQU	   "	$401A		; STKBOT
	;.EQU	   "	$401B		; STKBOT + 1
	;.EQU	   "	$401C		; STKEND
	;.EQU	   "	$401D		; STKEND + 1
	;.EQU	   "	$401E		; BREG
	;.EQU	   "	$401F		; MEM
	;.EQU		$4020		; MEM + 1
	;.EQU		$4021		; not used
	;.EQU		$4022		; DF_SZ
	;.EQU   	$4023		; S_TOP
	;.EQU		$4024		; S_TOP + 1

	; 4025/4026 - LASTK
	; 4027 - DBOUNCE
	; 4028 - MARGIN

	;.EQU		$4029		; NXTLIN
	;.EQU		$402A		; NXTLIN + 1
	;.EQU		$402B		; OLDPPC
	.EQU	METSPD	$402C		; OLDPPC + 1
	.EQU	PWRUPS	$402D		; FLAGX
	.EQU	METLEN	$402E		; STRLEN
	.EQU	METDIR	$402F		; STRLEN + 1
	
	; 4030/4031 - T_ADDR
	; 4032/4033 - SEED
	; 4034/4035 - FRAMES

	.EQU	METDLY	$4036		; COORDS-x
	.EQU	KEYDLY	$4037		; COORDS-y
	.EQU	SHIPPOS	$4038		; PR_CC
	.EQU	RSEED	$4039		; S_POSN-col
	;.EQU	   "	$403A		; S_POSN-lin

	; 403B - DFLAG

;*******************************************************************************
; Constants

	.EQU	KEYDELAY	6
	.EQU	METDELAY	8

	.EQU	PWR_BOMB	$01
	.EQU	BIT_BOMB	0

;*******************************************************************************
; Setup machine

	ld sp, $4400

; expand screen

	call EXPAND
	xor a				; Space for Bomb indicator
	ld (de),a
	inc de
	ld a, $76
	ld (de),a
	inc de
	ld a, $E9			; set jp (hl) as end of screen marker
	ld (de), a

;*******************************************************************************
; Setup new game

NEXTGAME

; Set Hi-Score

	ld hl, DFILE			; Score - 1
	ld de, DFILE + 10		; Hi-Sc - 1
	ld bc, 6			; length + 1
-	dec c			
	inc de
	inc hl
	ld a, (de)
	cp (hl)
	jr z, -				; if same, score still the same
	push hl
	call c, $A6F			; if higher, use ROM to copy

; Reset score

	ld hl, DFILE + 1
	ld b, 5
-	ld (hl), $1C
	inc hl
	djnz -

; Reset lives to 3 + 1

	inc hl
	inc hl
	ld (hl), $20			; '4'

	ld (iy + (PWRUPS - $4000)), 0
	ld (iy + (METDIR - $4000)), 0
	ld (iy + (METSPD - $4000)), METDELAY

;*******************************************************************************
; New life
NEXTLIFE

	; Reduce lives by 1

	ld hl,SCORE+7
	dec (hl)
	ld a, (hl)
	cp $1C				; '0'
	jr nz, +

	; No more lives

	ld hl, GAMEOVR
	ld de, DFILE + 1 + 96 + 3
	ld bc, 9
	ldir

	call WAITKEY
	jp NEXTGAME
+
	ld hl, ANYKEY
NL1	ld de, DFILE + 1 + 96 + 3
	ld bc, 9
	ldir

	ld a, (METDIR)
	cp 0
	push af
	call z, SCRLUP
	pop af
	call nz, SCRLDWN

	; Wait for user to start

	call WAITKEY

;*******************************************************************************

INITLIFE
	call CLRSCN
	ld a, (METSPD)
	ld (METDLY), a
	ld (iy + (METLEN - $4000)), 0
	ld (iy + (KEYDLY - $4000)), KEYDELAY

;*******************************************************************************
; Start playing game
GAMELOOP

; Scroll screen

	ld a, (SHIPPOS)				; Calc ship addr
	ld h, DFILE / 256
	ld l, a
	ld (hl), $80				; Clear ship
	
	; Delay for scrolling

	ld a, (METDLY)
	dec a
	jr z, +
	ld (METDLY), a
	jr MVSHIP
+
	ld a, (METSPD)
	ld (METDLY), a

SCROLL
	ld bc, 12 * 16
SCRL1	ld hl, SCRLTO+16			; From
SCRL2	ld de, SCRLTO				; To
SCRL3	ldir

	; Clear new line

	ld a, $80
	ld b, 15
-	ld (de), a
SCRL4	inc de
	djnz -

; Add a meteor

	call RND1			; Pick meteor
	and $0F
	ld hl, METEORS
	call PRNOBJ
	call INCSC

; Powerup

	call RND1
	cp $03
	jr nc, +			; No Power up
	ld a, $A7
	call PRNOBJ2
+
	; Reached end of storm

	ld bc, METLEN
	ld a, (BC)
	dec a
	ld (BC), a
	jr nz, MVSHIP

	ld a, (METSPD)			; Speed up storm
	dec a
	jr z, +
	ld (METSPD), a
+
	inc BC				; Change Direction
	ld a, (BC)
	xor $01
	ld (BC), a
	ld hl, MADEIT
	jp NL1

;*******************************************************************************
; Move Ship

MVSHIP
	ld de, PWRUPS
	ld a, (SHIPPOS)				; Calc ship addr
	ld h, DFILE / 256
	ld l, a

	; Delay for moving

	ld a, (KEYDLY)
	dec a
	jr z, +
	ld (KEYDLY), a
	jr DISPSHIP
+
	ld (iy + (KEYDLY - $4000)), KEYDELAY

; Check if user pressed a key
	ld a, (LASTK)
	bit 7, a				; Bomb:   B to spc
	jr nz, +

	ld a, (de)
	bit BIT_BOMB, a
	jr z, DISPSHIP				; Not collected
	and 255 - PWR_BOMB
	ld (de), a
	ld c, $00
	call CLRSCN1				; Needs delay
	ld bc, $1000
-	dec bc
	ld a, b
	or c
	jr nz, -
	call CLRSCN

	jr DISPSHIP
+
	bit 2, a				; Left:   Q to T
	jr nz, +

	dec hl
	ld a, (hl)
	cp $76
	jr nz, ++
	inc hl
	jr z, ++
+
	bit 5, a				; Right:  Y to P
	jr nz, DISPSHIP

	inc hl
	ld a, (hl)
	cp $76
	jr nz, ++
	dec hl
++
	ld a, l
	ld (SHIPPOS), a
	
;*******************************************************************************
; Display Ship

DISPSHIP

; Are we hit?

	ld a, (hl)
	cp $80
	jr z, PRSHIP				; not hit meteor

	cp $A7					; 'B'
	jr nz, +
	ld a, (PWRUPS)
	or PWR_BOMB
	ld (PWRUPS), a
	jr PRSHIP
+	
	cp $97					; '*'
	jp nz, HITMETR				; Hit meteor
	; Add 5 to score
	ld b, 5					; SC = SC + 5
-	call INCSC				; Rewrite?
	djnz -

PRSHIP
SCRL6	ld (hl), $BB 			; 'V'

	ld a, (PWRUPS)
	bit BIT_BOMB, a
	ld a, $00
	jr z, +					; Not collected
	ld a, $27
+
	ld (DFILE + 1+ (14 * 16)), a

;*******************************************************************************
; Busy wait for next frame

BSYWAIT
	ld hl, FRAMES
	ld a, (hl)
-	cp (hl)
	jr z, -
	
	jp GAMELOOP

;*******************************************************************************
; Routines

RND1 

; Joe Wingbermuehle Ion Random
; https://wikiti.brandonw.net/index.php?title=Z80_Routines:Math:Random

        ld      hl,(RSEED)
        ld      a,r
        ld      d,a
        ld      e,(hl)
        add     hl,de
        add     a,l
        xor     h
        ld      (RSEED),hl
        ret	

; Hit meteor

HITMETR
	; hl = Addr or player

	ld (hl), $17			; '*'
	ld b, 16
LP0	ld a, (hl)
	and $7f
	ld (hl), a
	ld de, $0800
-	dec de
	ld a, d
	or e
	jr nz, -
	ld a, (hl)
	or $80
	ld (hl), a
	ld de, $0800
-	dec de
	ld a, d
	or e
	jr nz, -
	djnz LP0
	jp NEXTLIFE

; inc scrore

INCSC
	push hl
	ld hl, SCORE + 5
	.DB $11				; LD DE op
-	ld (hl), $1C			; '0'
	dec hl
	inc (hl)
	ld a, (hl)
	cp $26				; > '9'
	jr z, -
	pop hl
	ret

WAITKEY
-	call RND1				; Create entropy
	ld a, (LASTK)
	cp 255
	jr z, -

; Wait for key release

-	call RND1				; Create entropy
	ld a, (LASTK)
	cp 255
	jr nz, -
	ret

SCRLUP
	ld a, 24
	ld (SHIPPOS), a

	ld a, $BB				; 'V'
	ld (SCRL6+1), a

	ld hl, SCRLTO+16			; From
	ld (SCRL1+1), hl
	ld hl, SCRLTO				; To
	ld (SCRL2+1), hl
	ld a, $B0				; ldir
	ld (SCRL3+1), a
	ld a, $13				; inc de
	ld (SCRL4), a

	ld a, 209
	ld (SCRL5+1), a

	ret

SCRLDWN
	ld a, 216
	ld (SHIPPOS), a

	ld a, $A6				; 'A'
	ld (SCRL6+1), a

	ld hl, SCRLTO+190			; From
	ld (SCRL1+1), hl
	ld hl, SCRLTO+206			; To
	ld (SCRL2+1), hl
	ld a, $B8				; lddr
	ld (SCRL3+1), a
	ld a, $1B				; dec de
	ld (SCRL4), a

	ld a, 17
	ld (SCRL5+1), a

	ret

PRNOBJ
	ld e, a
	ld d, 0
	add hl, de
	ld a, (hl)
PRNOBJ2
	push af
-	call RND1			; Print meteor
	and $0F
	cp $0F
	jr z, -

SCRL5	add a, 209
	ld h, DFILE / 256
	ld l, a
	pop af
	ld (HL), a
	ret

CLRSCN
	ld c, $80
CLRSCN1	push hl
	ld hl, DFILE + 1 + 16
	ld b, (16 * 13)
-	ld a, (hl)
	cp $76				; Skip $76
	jr z, +
	ld (hl), c
+	inc hl
	djnz -

	pop hl
	ret

;*******************************************************************************
; Data - Can any of this be moved to SYSVARS?

METEORS	.DB $01, $02, $03, $04, $05, $07, $9B, $97
	.DB $81, $82, $83, $84, $85, $87, $9B, $97

.ENDS
;*******************************************************************************

.BANK 1 SLOT 1
.ORGA $4300

.SECTION "DFILE"

; Display 15 x 14 = 1 + (16 * 14) + 1 = 226 bytes
; 14 lines leaves 30 bytes for stack

DFILE	.DB $76
SCORE	.ASC "00000  0  00000"
	.DB $76
SCRLTO	.DS 15, $80
	.DB $76
EXPAND
	ld hl, DATA				; Copy data to sysvars
	ld de, GAMEOVR
	ld bc, 29
	ldir

	ld hl, SCRLTO
	ld de, SCRLTO+16
	ld bc, 12*16
	push HL
	jp $0A6F				; use ROM LDIR to copy

DATA	.ASC "game_over"
	.DW $4300				; DFILE
	.ASC "_any_key_"
	.ASC "_made_it_"

VARAREA	.DB $80					; becomes end of screen

.ENDS
